package cn.kdan.cloud.pdf.office.account.rabbit.listener;

import cn.kdan.cloud.pdf.office.account.model.User;
import cn.kdan.cloud.pdf.office.account.service.UserService;
import cn.kdan.cloud.pdf.office.account.service.impl.UserServiceImpl;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import com.rabbitmq.client.Channel;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.Map;
import java.util.Objects;

/**
 * @author ComPDFKit-WPH 2023/1/11
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class LogOffListener {

    private final UserService userService;


    /**
     * 注销用户 监听
     *
     * @param message    message
     * @param channel    channel
     * @param userId userId
     */
    @RabbitListener(queues = RabbitMqConstant.USER_LOGOFF_DELAY_QUEUE)
    public void backgroundApiAddQueue(Message message, Channel channel, String userId) {
        try {
            User user = userService.getUser(userId);
            //用户不为空并且用户的状态等于自动注销
            if(!ObjectUtils.isEmpty(user)
                    &&!StringUtils.isEmpty(user.getValidFlag())
                    &&PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString().equals(user.getValidFlag())){
                userService.handleLogOff(user,"三天用户自动注销");
            }
            channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("消息消费失败:" + e.getMessage(), e);
            throw new BackendRuntimeException("用户注销消息消费失败：userId：" + userId);
        }
    }


}
