package cn.kdan.cloud.pdf.office.api.account.feign;



import cn.kdan.cloud.pdf.office.api.account.feign.hystrix.UserSubscriptionInfoHystrix;
import cn.kdan.cloud.pdf.office.api.account.vo.UserSubscriptionInfo;
import cn.kdan.cloud.pdf.office.common.config.FeignConfig;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;

import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;


/**
 * @author tangxiangan
 */
@Component
@FeignClient(value = CommonConstant.ACCOUNT_PROVIDER_NAME + "/userSubscriptionInfo" , configuration = FeignConfig.class,fallback = UserSubscriptionInfoHystrix.class)
public interface UserSubscriptionInfoApi {

    /**
     * 登录后关联产品
     *
     * @param userSubscriptionInfo dto
     */
    @PostMapping(value = "insert")
    ResultMap<Boolean> insert(@RequestBody UserSubscriptionInfo userSubscriptionInfo);

    /**
     * 更新订阅信息（ex：用户取消订阅后修改订阅信息中的自动订阅）
     * @param userSubscriptionInfo dto
     * @return Boolean
     */
    @PostMapping("/update")
    ResultMap<Boolean> update(@RequestBody UserSubscriptionInfo userSubscriptionInfo);

    /**
     * 根据用户id和平台获取他的产品使用信息
     * @param userId 用户id
     * @param platform 平台
     * @return ResultMap
     */
    @GetMapping("/getByUserIdAndPlatform")
    ResultMap<UserSubscriptionInfo> getByUserIdAndPlatform(@RequestParam("userId") String userId, @RequestParam("platform") Integer platform);

    /**
     * 根据用户id和平台获取他的产品使用信息
     * @param userId 用户id
     * @param productId productId
     * @return UserSubscriptionInfo
     */
    @GetMapping("/getByUserIdAndProductId")
    ResultMap<UserSubscriptionInfo> getByUserIdAndProductId(@RequestParam("userId") String userId,@RequestParam("productId") String productId);

    /**
     * 获取用户在某个产品的所有会员
     * @param userId 用户id
     * @param appId 产品id
     * @return List
     */
    @GetMapping("/getByUserIdAndAppId")
    ResultMap<List<UserSubscriptionInfoVO>> getByUserIdAndAppId(@RequestParam("userId") String userId, @RequestParam("appId") String appId);
}
