package cn.kdan.cloud.pdf.office.api.account.utils;

import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.util.CollectionUtils;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/2/8
 * <p>
 * SubscriptionUtil
 */
@Slf4j
public class SubscriptionUtil {

    private SubscriptionUtil() {
    }

    /**
     * 订阅过期消息发送
     *
     * @param rabbitTemplate rabbitTemplate
     * @param subscriptionId subscriptionId
     * @param endTime        ttl
     */
    public static void subscriptionExpiredSendNews(RabbitTemplate rabbitTemplate, String subscriptionId, LocalDateTime endTime) {
        LocalDateTime now = LocalDateTime.now();
        long ttl = Duration.between(now, endTime).toMillis() + 1000L;
        if (ttl > 4294967295L) {
            // 设置30天
            ttl  = 2592000000L;
        }
        log.info("rabbitMq 发送延时队列，订阅过期时间：{}", ttl);
        // 订阅过期
        log.info("rabbitMq 发送延时队列，订阅过期：{}", subscriptionId);
        long finalTtl = ttl;
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                RabbitMqConstant.PAYMENT_SUBSCRIPTION_EXPIRED_ROUTING_KEY,
                subscriptionId,
                (msg -> {
                    msg.getMessageProperties().setHeader("x-delay", finalTtl + "");
                    return msg;
                }));
    }


    /**
     * 获取当前平台的订阅信息
     * 为null代表没有当前订阅信息
     *
     * @param userSubscriptionInfoVOS userSubscriptionInfoVOS
     * @param platform                platform
     * @return UserSubscriptionInfoVO
     */
    public static UserSubscriptionInfoVO getUserSubscriptionInfoByPlatform(List<UserSubscriptionInfoVO> userSubscriptionInfoVOS,
                                                                           Integer platform) {
        if (CollectionUtils.isEmpty(userSubscriptionInfoVOS)) {
            return null;
        }
        for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOS) {
            if (userSubscriptionInfoVO.getPlatform().equals(platform)) {
                return userSubscriptionInfoVO;
            }
        }
        return null;
    }

}
