package cn.kdan.cloud.pdf.office.api.product.feign;

import cn.kdan.cloud.pdf.office.api.product.feign.hystrix.ProductHystrix;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductPriceVo;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


@Component
@FeignClient(value = CommonConstant.PRODUCT_PROVIDER_NAME + "/product", fallback = ProductHystrix.class)
public interface ProductApi {


    @GetMapping("/getProductPrice")
    ResultMap<ProductPriceVo> getProductPrice(@NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                              @NotNull(message = "userId不能为空") @RequestParam("userId") String userId);


    @GetMapping("/getProduct")
    ResultMap<ProductVO> getProduct(@NotNull(message = "产品id不能为空") @RequestParam("productId") String productId);

    @GetMapping("/getProductForPay")
    ResultMap<ProductVO> getProductForPay(@NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                    @NotNull(message = "支付方式不能为空") @RequestParam("payment") Integer payment,
                                    @NotNull(message = "是否首年优惠不能为空") @RequestParam("isDiscount") Integer isDiscount);

    @GetMapping("/getProductList")
    ResultMap<IPage<ProductVO>> getProductList(@RequestParam("size") Long size,
                                               @RequestParam("page") Long page,
                                               @RequestParam("appId") String appId);

    /**
     * 获取新用户专用活动
     *
     * @param appId appId
     * @return 新用户专用活动
     */
    @GetMapping("/getNewUserProduct")
    ResultMap<List<ProductVO>> getNewUserProduct(@RequestParam("appId") String appId);

    @GetMapping("/getProductByAppIdAndModel")
    ResultMap<ProductVO> getProductByAppIdAndModel(@RequestParam("appId") String appId,@RequestParam("platform") Integer platform);

    @GetMapping("/getProductByThirdPayId")
    ResultMap<ProductVO> getProductByThirdPayId(@RequestParam("thirdPayId") String thirdPayId);

    @GetMapping("/getListingProducts")
    ResultMap<List<ListingProductVO>> getListingProducts(@Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation,
                                                                @Validated @NotNull(message = "平台id不能为空") @RequestParam("platformId") String platformId);

    @GetMapping("/getProductsByIds")
    ResultMap<List<ListingProductVO>> getProductsByIds(@RequestParam(value = "productIds", required = false) List<String> productIds);

    /**
     * 验证用户邮箱教育优惠资格
     * @param email 用户邮箱
     * @return
     */
    @GetMapping("/checkEmail")
    ResultMap<Integer> checkEmail(@RequestParam("email") String email);

    @GetMapping("/getProductPriceForBuy")
    ResultMap<ListingProductVO> getProductPriceForBuy(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                             @Validated @NotNull(message = "用户id不能为空") @RequestParam("userId") String userId,
                                                             @Validated @NotNull(message = "是否教育优惠不能为空") @RequestParam("isEducation") Integer isEducation);

    @GetMapping("/getBatchProductPrice")
    ResultMap<BigDecimal> getBatchProductPrice(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                      @Validated @NotNull(message = "购买数量不能为空") @Min(2) @RequestParam("num") Integer num);

    @GetMapping("/checkCoupon")
    ResultMap<ListingProductVO> checkCoupon(@Validated @NotNull(message = "产品id不能为空") @RequestParam("productId") String productId,
                                                   @NotNull(message = "用户id不能为空") @RequestParam("userId") String userId,
                                                   @NotNull(message = "券码不能为空") @RequestParam("code") String code);

    @GetMapping("/getProductByCode")
    ResultMap<ProductVO> getProductByCode(@RequestParam("code") String code);

    @PostMapping("/updateCouponState")
    ResultMap<Boolean> updateCouponState(@Validated @NotNull(message = "券码不能为空") @RequestParam("code") String code,
                                                @Validated @NotNull(message = "状态不能为空") @RequestParam("state") Integer state,
                                         @Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId);

    @PostMapping("/refundCouponByOrderId")
    ResultMap<Boolean> refundCouponByOrderId(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId);
}
