package cn.kdan.cloud.pdf.office.email;

import cn.kdan.cloud.pdf.office.api.email.bo.EmailSendBO;
import cn.kdan.cloud.pdf.office.common.enums.EmailCodeTypeEnum;
import cn.kdan.cloud.pdf.office.email.service.EmailLogService;
import cn.kdan.cloud.pdf.office.email.service.SendEmailHandlerService;
import cn.kdan.cloud.pdf.office.email.utils.TemplatesUtil;
//import com.itextpdf.kernel.pdf.PdfDocument;
//import com.itextpdf.kernel.pdf.PdfWriter;
//import org.apache.commons.lang3.StringUtils;
import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StreamUtils;
import org.springframework.web.client.RestTemplate;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


/**
 * @author ComPDFKit-WPH 2023/1/11
 */
@SpringBootTest
public class SendEmailTest {

    @Autowired
    private EmailLogService emailLogService;
    @Autowired
    private SendEmailHandlerService emailHandlerService;

    private static final RestTemplate restTemplate = new RestTemplate();

    @Test
    public void sendTest(){
        // 获取模板
        // 获取模板
        try {
            String invoiceHtml = new String(StreamUtils.copyToByteArray(SendEmailHandlerService.class.getClassLoader().getResourceAsStream("templates/Invoice.html")), StandardCharsets.UTF_8);
            System.out.println(invoiceHtml);
            // 填充字段
            Map<String, Object> map = new HashMap<>();
            map.put("email", "email");
            map.put("orderNumber", "orderNumber");
            map.put("purchasedDate", "purchasedDate");
            map.put("invoiceDate", "invoiceDate");
            map.put("invoiceNumber", "invoiceNumber");
            map.put("productName", "productName");
            map.put("unitPrice", "unitPrice");
            map.put("discount", "discount");
            map.put("amount", "amount");
            map.put("total", "total");
            String htmlStr = TemplatesUtil.replaceStringUsingFreeMarker(invoiceHtml, map);
            // 生成发票,  C端发票操作
            // 设置请求header
            HttpHeaders headers = new HttpHeaders();
            headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
            // 封装表单数据
            MultiValueMap<String, String> paramMap = new LinkedMultiValueMap<>();
            paramMap.add("html", htmlStr);
            HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(paramMap, headers);
            // 发送POST请求
            Map postForObjectMap = restTemplate.postForObject("http://139.196.160.101:3060/api/get-invoice", request, Map.class);
            String invoiceFileUrl = postForObjectMap.get("url").toString();
            System.out.println(invoiceFileUrl);
            EmailSendBO bo = new EmailSendBO();
            bo.setToEmail("316531990@qq.com");
            Map<String,String> contentMap = new HashMap<>();
            contentMap.put("@order_number@","order_number");
            contentMap.put("@product@", "product");
            contentMap.put("@subscription@","yearly plan");
            DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd hh:mm");
            contentMap.put("@purchase_date@",dateFormat.format(new Date()));
            contentMap.put("@price@","price");
            contentMap.put("@expiration_date@",dateFormat.format(new Date()));
            bo.setSendContent(contentMap);
            //设置内容 @order_number@ @product@ @subscription@ @purchase_date@  @price@ @exiration_date@
            bo.setUseEvent(EmailCodeTypeEnum.SUBSCRIPTION_SUCCESS.value());
            //设置title
            Map<String,String> titleMap = new HashMap<>();
            bo.setSendTitleContent(titleMap);
            bo.setFileUrl(invoiceFileUrl);
            bo.setFileName("invoice.pdf");
            emailHandlerService.sendEmail(bo);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

}
