package cn.kdan.cloud.pdf.office.payment.controller;

import cn.kdan.cloud.pdf.office.api.payment.vo.OrderGiftLogVO;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.payment.client.GooglePayClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGift;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGiftLog;
import cn.kdan.cloud.pdf.office.payment.service.*;
import cn.kdan.cloud.pdf.office.payment.webhook.google.SubscriptionPurchaseV3;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.RequiredArgsConstructor;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import javax.validation.constraints.NotNull;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author ComPDFKit-WPH 2023/2/6
 * <p>
 * OrderController
 */
@RestController
@RequestMapping("/order")
@RequiredArgsConstructor
public class OrderController {

    private final OrderService ordersService;

    private final SubscriptionsService subscriptionsService;

    private final ProductApi productApi;

    private final WebhookService webhookService;

    private final AppStoreService appStoreService;
    private final GooglePayService googlePayService;

    private final OrderGiftService orderGiftService;
    private final OrderGiftLogService orderGiftLogService;

    private final RestTemplate restTemplate = new RestTemplate();
    private final RedissonClient redissonClient;
    private final GooglePayClient googlePayClient;

    /**
     * 创建支付订单
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/createOrder")
    public ResultMap<CreateOrderResultVO> createOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO){
        return ResultMap.success(ordersService.createOrder(createOrderDTO));
    }

    @PostMapping("/createUserOrder")
    public ResultMap<Map<String, String>> createUserOrder(@Validated @RequestBody CreateUserOrderDTO createUserOrderDTO) {
        return ResultMap.success(ordersService.createUserOrder(createUserOrderDTO));
    }

    @PostMapping("/createUserSubscription")
    public ResultMap<Map<String, String>> createUserSubscription(@Validated @RequestBody CreateUserOrderDTO createUserOrderDTO) {
        return ResultMap.success(ordersService.createUserSubscription(createUserOrderDTO));
    }

    /**
     *  权益校验 （会员系统）
     *
     * @param  equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @PostMapping("/appStoreEquityVerification")
    public ResultMap<UserInfoVO> appStoreEquityVerification(@RequestBody EquityVerificationDTO equityVerificationDTO){
        return ResultMap.success(appStoreService.appStoreEquityVerification(equityVerificationDTO));
    }

    /**
     * 谷歌 权益校验 （会员系统）
     *
     * @param googlePayDTO googlePayDTO
     * @return UserInfoVO
     */
    @PostMapping("/googleEquityVerification")
    public ResultMap<UserInfoVO> googleEquityVerification(@RequestBody GooglePayDTO googlePayDTO){
        return ResultMap.success(googlePayService.googleEquityVerification(googlePayDTO));
    }

    @PostMapping("/updateStatusPayment")
    public ResultMap<Boolean> updateStatusPayment(@RequestParam String userId, @RequestParam String productId){
        updatePayment(userId,productId);
        return ResultMap.success(true);
    }

    public void updatePayment(String userId, String productId) {
        /*
         如果用户有进行中的订阅，查最近的完成订阅订单，
         */
        Order order = ordersService.getOrderByUserIdAndProductId(userId, productId);
        //如果没有订单结束
        if(ObjectUtils.isEmpty(order)||StringUtils.isEmpty(order.getResult())){
            return;
        }
        if(order.getPayment()==6){
            SubscriptionPurchaseV3 subscriptionGoogleOrderV2 = googlePayClient.getSubscription("com.pdftechnologies.pdfreaderpro",order.getResult());
            if(subscriptionGoogleOrderV2.getSubscriptionState().equals("SUBSCRIPTION_STATE_CANCELED")||subscriptionGoogleOrderV2.getSubscriptionState().equals("SUBSCRIPTION_STATE_EXPIRED")){
                googlePayService.subscriptionCancel(subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId());
            }
            if(subscriptionGoogleOrderV2.getSubscriptionState().equals("SUBSCRIPTION_STATE_ACTIVE")){
                String lockKey = Integer.toString(2).concat(subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId()) + "-RedissonLock";
                RLock rLock = redissonClient.getLock(lockKey);
                try {
                    // 上锁
                    boolean resultLock = rLock.tryLock(30, 10, TimeUnit.SECONDS);
                    if (resultLock) {
                        googlePayService.subscriptionPaymentSucceeded(false, subscriptionGoogleOrderV2.getExternalAccountIdentifiers().getObfuscatedExternalAccountId(), subscriptionGoogleOrderV2.getLatestOrderId(), 2, subscriptionGoogleOrderV2);
                    }
                } catch (InterruptedException e) {

                } finally {
                    if (rLock.isLocked() && rLock.isHeldByCurrentThread()) {
                        rLock.unlock();
                    }
                }
            }
        }
    }

    /**
     * 手动创建虚拟订单
     *
     * @param createOrderManual createOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/createOrderManual")
    public ResultMap<OrderUpdateVO> createOrderManual(@Validated @RequestBody CreateOrderManualDTO createOrderManual){
        return ResultMap.success(ordersService.createOrderManual(createOrderManual));
    }

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    public ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                            @RequestParam("orderId") String orderId){
        return ResultMap.success(ordersService.getOrderById(orderId));
    }

    @GetMapping("/getStateByOrderId")
    ResultMap<Map<String, String>> getStateByOrderId(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        return ResultMap.success(ordersService.getStateByOrderId(orderId));
    }

    @GetMapping("/getOrderListByStatus")
    ResultMap<List<OrdersVO>> getOrderListByStatus(@Validated @NotNull(message = "订单状态不能为空") @RequestParam("orderStatus") Integer orderStatus,
                                                   @Validated @NotNull(message = "userId不能为空") @RequestParam("userId") String userId) {
        return ResultMap.success(ordersService.getOrderListByStatus(orderStatus, userId));
    }

    @PostMapping("/closeOrder")
    ResultMap<Boolean> closeOrder(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        ordersService.closeOrder(orderId);
        return ResultMap.success(true);
    }

    /**
     * 查询订单详情（用于判断订阅是否已经支付完成）
     * @param orderId
     * @return OrdersVO
     */
    @GetMapping("/getOrderWithUpdate")
    public ResultMap<OrdersVO> getOrderWithUpdate(@Validated @NotNull(message = "orderId不能为空")
                                            @RequestParam("orderId") String orderId){
        return ResultMap.success(webhookService.getOrderWithUpdate(orderId));
    }

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    public ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                                 @RequestParam("tradeNo") String tradeNo){
        return ResultMap.success(ordersService.getOrderByTradeNo(tradeNo));
    }

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    public ResultMap<Void> chargebackOrder(@Validated @RequestBody ChargebackOrderDTO chargebackOrderDTO){
        // 修改 订单状态
        ordersService.chargebackOrder(chargebackOrderDTO);
        // 修改 订阅状态
        subscriptionsService.chargebackSubscription(chargebackOrderDTO);
        return ResultMap.success();
    }

    /**
     * 取消续订
     *
     * @param cancelSubscription cancelSubscription
     * @return Void
     */
    @PostMapping("/cancelSubscription")
    public ResultMap<Void> cancelSubscription(@Validated @RequestBody CancelSubscriptionDTO cancelSubscription) {
        subscriptionsService.chargebackSubscription(cancelSubscription);
        return ResultMap.success();
    }

    /**
     * 获取会员系统用户订单
     * @param userId
     * @return
     */
    @GetMapping("/getOrderByUserId")
    ResultMap<List<OrdersVO>> getOrderByUserId(@Validated @NotNull(message = "userId不能为空")
                                               @RequestParam("userId") String userId) {
        return ResultMap.success(ordersService.getOrderByUserId(userId, "1"));
    }

    @PostMapping("/giveAway")
    public ResultMap<OrderGiftLogVO> giveAway(@RequestParam("userId") String userId, @RequestParam("email") String email, @RequestParam("name") String name) {
        OrderGift orderGift = orderGiftService.giveAway(userId);
        OrderGiftLog orderGiftLog = orderGiftLogService.logGift(userId,email,name,orderGift.getId());
        OrderGiftLogVO result = new OrderGiftLogVO();
        BeanUtils.copyProperties(orderGiftLog,result);
        return ResultMap.success(result);
    }
    /**
     * 根据用户id查询list可赠送的数量
     */
    @GetMapping("/getGiftNum")
    public ResultMap<Long> getGiftNum(@RequestParam("userId") String userId) {
        return ResultMap.success(orderGiftService.getGiftNum(userId));
    }

    @GetMapping("/getOrderGiftLogById")
    public ResultMap<OrderGiftLogVO> getOrderGiftLogById(@RequestParam("orderGiftLogId") String orderGiftLogId) {
        OrderGiftLogVO result = new OrderGiftLogVO();
        OrderGiftLog orderGiftLog = orderGiftLogService.getById(orderGiftLogId);
        BeanUtils.copyProperties(orderGiftLog,result);
        return ResultMap.success(result);
    }

    /**
     * 接收
     * @param orderGiftLogId  订单赠送记录id
     * @param receiveUserId 接收者id
     * @return
     */
    @PostMapping("/acceptUser")
    public ResultMap<Boolean> acceptUser(@RequestParam("orderGiftLogId") String orderGiftLogId,@RequestParam("receiveUserId") String receiveUserId) {
        return ResultMap.success(orderGiftLogService.acceptUser(orderGiftLogId,receiveUserId));
    }
    //分页查询orderGiftLog
    @GetMapping("/pageOrderGiftLog")
    public ResultMap<Page<OrderGiftLogVO>> pageOrderGiftLog(@RequestParam("userId")String userId,
                                                             @RequestParam(name = "email",required = false)String email,
                                                             @RequestParam(name = "name",required = false)String name,
                                                             @RequestParam(name = "page",defaultValue = "1",required = false)Integer page,
                                                             @RequestParam(name ="pageSize",defaultValue = "10",required = false)Integer pageSize) {
        IPage<OrderGiftLog> pageGiftLog = orderGiftLogService.orderGiftLogPage(userId, email, name, page, pageSize);
        List<OrderGiftLogVO> vos = new ArrayList<>();
        pageGiftLog.getRecords().stream().forEach(orderGiftLog -> {
            OrderGiftLogVO vo = new OrderGiftLogVO();
            BeanUtils.copyProperties(orderGiftLog,vo);
            //localdatetime转date
            vo.setCreatedAt( Date.from(orderGiftLog.getCreatedAt().atZone(ZoneId.systemDefault()).toInstant()));
            vo.setUpdatedAt(Date.from(orderGiftLog.getUpdatedAt().atZone(ZoneId.systemDefault()).toInstant()));
            vos.add(vo);
        });
        Page<OrderGiftLogVO> resultPage = new Page<>(pageGiftLog.getPages(), pageGiftLog.getSize());
        resultPage.setTotal(pageGiftLog.getTotal());
        resultPage.setRecords(vos);
        return ResultMap.success(resultPage);
    }
}
