package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.payment.service.*;
import cn.kdan.cloud.pdf.office.payment.webhook.appstore.notification.JWSTransactionDecodedPayload;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author ComPDFKit-WPH 2023/1/30
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class AppStoreWebhookServiceImpl implements AppStoreWebhookService {

    private final OrderService ordersService;
    private final WebhookService webhookService;

    @Override
    public void subscriptionPaymentSucceeded(Boolean isDiscount, String thirdTradeNo, String thirdOrderId, Integer payTime, JWSTransactionDecodedPayload jwsTransactionDecodedPayload) {
        log.info("appstore 订阅的支付订单已成功 appstore订单id：{}", thirdOrderId);
        OrdersVO ordersVO = ordersService.getOrderByThirdOrderNo(thirdOrderId);
        //如果处理过
        if (ObjectUtils.isNotEmpty(ordersVO) && OrderConstant.COMPLETED.equals(ordersVO.getStatus())) {
            log.info("appstore 订阅的续费支付订单已经处理过 appstore订单id：{}", thirdOrderId);
            return;
        }
        List<OrdersVO> ordersVOS = ordersService.getOrderByTradeNo(thirdTradeNo);
        if (CollectionUtils.isEmpty(ordersVOS)){
            log.info("appstore 订阅的续费支付订单没有初始订阅 appstore订单id：{}", thirdOrderId);
            return;
        }
        OrdersVO ordersVONew = ordersVOS.get(0);
        BigDecimal price = BigDecimal.valueOf(jwsTransactionDecodedPayload.getPrice()/1000);
        webhookService.handleSubsequentAutomaticDeduction(price, thirdTradeNo, thirdOrderId,ordersVONew,payTime, jwsTransactionDecodedPayload.toString());
    }

    @Override
    public void subscriptionCancel(String thirdSubscriptionId) {
        log.info("订阅取消续订，appstore订阅id：{}", thirdSubscriptionId);
        // 根据第三放订阅id查询对应的用户id和产品id
        webhookService.handleMemberUnsubscribeFollowUpActions(thirdSubscriptionId);
    }
}
