package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.OrderGiftLogVaildFlagEnum;
import cn.kdan.cloud.pdf.office.common.enums.OrderGiftVaildFlagEnum;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGift;
import cn.kdan.cloud.pdf.office.payment.entity.OrderGiftLog;
import cn.kdan.cloud.pdf.office.payment.mapper.OrderGiftLogMapper;
import cn.kdan.cloud.pdf.office.payment.mapper.OrderGiftMapper;
import cn.kdan.cloud.pdf.office.payment.service.OrderGiftLogService;
import cn.kdan.cloud.pdf.office.payment.utils.CommonUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @author ComPDFKit-WPH 2024-10-28
 */

@Service
public class OrderGiftLogServiceImpl extends ServiceImpl<OrderGiftLogMapper, OrderGiftLog> implements OrderGiftLogService {
    @Autowired
    private OrderGiftMapper orderGiftMapper;
    @Autowired
    private RabbitTemplate rabbitTemplate;


    @Override
    public IPage<OrderGiftLog> page(OrderGiftLog query) {
        return this.page(new Page<>(query.getPage(), query.getSize()), Wrappers.query(query));
    }

    @Override
    public OrderGiftLog logGift(String userId, String email, String name, String orderGiftId) {
       //根据赠送者id userId,接受者邮箱，接受者name，插入记录表，状态为"待接收"
        String id = CommonUtils.generateId();
        OrderGiftLog orderGiftLog = new OrderGiftLog();
        orderGiftLog.setId(id);
        orderGiftLog.setUserId(userId);
        orderGiftLog.setEmail(email);
        orderGiftLog.setName(name);
        orderGiftLog.setOrderGiftId(orderGiftId);
        orderGiftLog.setValidFlag(OrderGiftLogVaildFlagEnum.RECEIVE_GIFT.getCode());
        this.save(orderGiftLog);
        rabbitTemplate.convertAndSend(RabbitMqConstant.ORDER_GIFT_DELAY_EXCHANGE,
                RabbitMqConstant.ORDER_GIFT_DELAY_ROUTING_KEY,
                id,
                (msg -> {
                    //msg.getMessageProperties().setHeader("x-delay", (24 * 60 * 60 * 1000) + "");
                    msg.getMessageProperties().setHeader("x-delay", (24 * 60 * 60 * 1000) + "");
                    return msg;
                }));
        return orderGiftLog;
    }

    @Override
    public void expire(String orderGiftId) {
        this.update(Wrappers.<OrderGiftLog>lambdaUpdate().set(OrderGiftLog::getValidFlag, OrderGiftLogVaildFlagEnum.RECEIVE_FAIL.getCode()).eq(OrderGiftLog::getOrderGiftId, orderGiftId));
        //并且更新orderGift表中对应记录的validFlag
        orderGiftMapper.update(null, Wrappers.<OrderGift>lambdaUpdate().set(OrderGift::getValidFlag, OrderGiftVaildFlagEnum.GIFT_ABLE.getCode()).eq(OrderGift::getId, orderGiftId));
    }
    //接收会员
    @Override
    public Boolean acceptUser(String orderGiftLogId,String receiveUserId) {
        OrderGiftLog orderGiftLog = this.getById(orderGiftLogId);
        if (orderGiftLog == null) {
            return false;
        }
        //更新状态为已接收
        this.update(Wrappers.<OrderGiftLog>lambdaUpdate().
                set(OrderGiftLog::getValidFlag, OrderGiftLogVaildFlagEnum.RECEIVED.getCode()).
                set(OrderGiftLog::getReceiveAt, new Date())
                .eq(OrderGiftLog::getId, orderGiftLogId));
        //并且更新orderGift表中对应记录的validFlag
        orderGiftMapper.update(null, Wrappers.<OrderGift>lambdaUpdate().set(OrderGift::getValidFlag, OrderGiftVaildFlagEnum.GIFTED.getCode())
                .set(OrderGift::getReceiveUserId, receiveUserId)
                .eq(OrderGift::getId, orderGiftLog.getOrderGiftId()));
        return true;
    }
    //根据id查询
    @Override
    public OrderGiftLog getById(String id) {
        return this.getOne(Wrappers.<OrderGiftLog>lambdaQuery().eq(OrderGiftLog::getId, id));
    }

    //分页查询 用户id 用户邮箱 用户名
    @Override
    public IPage<OrderGiftLog> orderGiftLogPage(String userId, String email, String name, Integer page, Integer pageSize) {
        //如果email不为空才查email
        //如果name不为空才查名字
        return this.page(new Page<>(page, pageSize), Wrappers.<OrderGiftLog>lambdaQuery().eq(OrderGiftLog::getUserId, userId)
                .eq(StringUtils.isNotEmpty(email), OrderGiftLog::getEmail, email)
                .eq(StringUtils.isNotEmpty(name), OrderGiftLog::getName, name));
    }
}
