package cn.kdan.cloud.pdf.office.payment.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.bo.*;
import cn.kdan.cloud.pdf.office.api.payment.constant.OrderConstant;
import cn.kdan.cloud.pdf.office.api.payment.constant.UserAction;
import cn.kdan.cloud.pdf.office.api.payment.dto.ChargebackOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateOrderManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.enums.ProductTypeEnum;
import cn.kdan.cloud.pdf.office.api.payment.paypal.ApplicationContext;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.constant.RedisKeyConstant;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.MyDateUtils;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.payment.client.PaddleClient;
import cn.kdan.cloud.pdf.office.payment.client.PaypalClient;
import cn.kdan.cloud.pdf.office.payment.entity.Order;
import cn.kdan.cloud.pdf.office.payment.error.ErrorMessage;
import cn.kdan.cloud.pdf.office.payment.mapper.OrderMapper;
import cn.kdan.cloud.pdf.office.payment.properties.PayCenterConfigProperties;
import cn.kdan.cloud.pdf.office.payment.properties.PaypalProperties;
import cn.kdan.cloud.pdf.office.payment.service.OrderService;
import cn.kdan.cloud.pdf.office.payment.utils.CommonUtils;
import cn.kdan.cloud.pdf.office.payment.utils.RSAUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.UuidUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.jetbrains.annotations.NotNull;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.cache.RedisCache;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023-01-29
 * <p>
 * OrdersServiceImpl
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements OrderService {

    private final ProductApi productApi;

    private final PaddleClient paddleClient;

    private final RabbitTemplate rabbitTemplate;


    private final PaypalClient paypalClient;

    private final PaypalProperties paypalProperties;

    private final UserApi userApi;

    private final PayCenterConfigProperties payCenterConfigProperties;


    @Resource(name = "myRedisCacheManager")
    private RedisCacheManager myRedisCacheManager;

    @Override
    @Transactional
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#3#h",
            key = "T(String).valueOf(#createOrderDTO.email)" +
                    ".concat('-').concat(#createOrderDTO.productId)" +
                    ".concat('-').concat(#createOrderDTO.paymentMethod.getValue())",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public CreateOrderResultVO createOrder(CreateOrderDTO createOrderDTO) {
        //判断是否有首年优惠
        ProductVO product = productApi.getProductForPay(createOrderDTO.getProductId(), createOrderDTO.getPaymentMethod().getValue(), createOrderDTO.getIsDiscount()).getResult();
        PaymentMethodEnum paymentMethodEnum = createOrderDTO.getPaymentMethod();
        log.info("createOrder:{}", createOrderDTO);
        Order order = new Order();
        order.setAppId(product.getAppId().toString());
        if (StringUtils.isNotEmpty(createOrderDTO.getUserId())) {
            order.setUserId(createOrderDTO.getUserId());
        }
        order.setEmail(createOrderDTO.getEmail());
        order.setProductId(createOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
        String readeNo = MyDateUtils.getTimeStamp() + "-" + createOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        switch (paymentMethodEnum) {
            case WXPAY:
                break;
            case ALIPAY:
                break;
            case PADDLE:
                //设置折扣价格
                BigDecimal price = createOrderDTO.getIsDiscount() == 0 ? product.getPrice() : product.getDisplayPrice();
                GeneratePayLinkPaddleBO paddleBO = new GeneratePayLinkPaddleBO();
                paddleBO.setPrices(new String[]{"USD:" + price.toString()});
                paddleBO.setProductId(product.getThirdPayId());
                paddleBO.setPassthrough(readeNo);
                if (StringUtils.isNotEmpty(createOrderDTO.getCountryCode())) {
                    paddleBO.setCustomerCountry(createOrderDTO.getCountryCode());
                }
                if (StringUtils.isNotEmpty(createOrderDTO.getEmail())) {
                    paddleBO.setCustomerEmail(createOrderDTO.getEmail());
                }
                if (StringUtils.isNotEmpty(createOrderDTO.getPostCode())) {
                    paddleBO.setCustomerPostcode(createOrderDTO.getPostCode());
                }
                GeneratePayLinkResultPaddleBO resultPaddleBO = paddleClient.createSubscriptions(paddleBO);
                order.setPagePayUrl(resultPaddleBO.getUrl());
                order.setResult(paddleBO.toString());
                order.setThirdOrderNo(paddleBO.getPassthrough());
                //订单类型1网站购买
                order.setType(1);
                //设置第三方订单
                order.setThirdTradeNo(paddleBO.getPassthrough());
                //设置支付方式
                order.setPayment(paymentMethodEnum.getValue());
                break;
            case NO_NEED_TO_PAY:
                break;
            case PAYPAL:
                //创建paypal订阅
                if (ProductTypeEnum.PLAN.getValue().equals(product.getType())) {
                    CreateSubscriptionsPaypalBO createSubscriptionsBO = new CreateSubscriptionsPaypalBO();
                    ApplicationContext applicationContext = new ApplicationContext();
                    applicationContext.setBrand_name("KDanMobile");
                    applicationContext.setLocale("zh-CN");
                    applicationContext.setUser_action(UserAction.SUBSCRIBE_NOW);
                    //成功或失败回调url
                    applicationContext.setReturn_url(paypalProperties.getReturnUrl());
                    applicationContext.setCancel_url(paypalProperties.getCancelUrl());
                    //订阅计划id
                    createSubscriptionsBO.setPlan_id(product.getThirdPayId());
                    createSubscriptionsBO.setApplication_context(applicationContext);
                    CreateSubscriptionsResultPaypalBO resultBO = paypalClient.createSubscriptions(createSubscriptionsBO);
                    order.setResult(resultBO.toString());
                    order.setThirdOrderNo(resultBO.getId());
                    //订单类型1网站购买
                    order.setType(1);
                    //设置第三方订单
                    order.setThirdTradeNo(resultBO.getId());
                    //设置支付方式
                    order.setPayment(paymentMethodEnum.getValue());
                    order.setPagePayUrl(resultBO.getLinks().get(0).getHref());
                }
                break;
            // 当前支付方式无效
            default:
                throw new BackendRuntimeException(ErrorMessage.PAYMENT_METHOD_INVALID);
        }
        this.baseMapper.insert(order);
        CreateOrderResultVO createOrderResultVO = new CreateOrderResultVO();
        BeanUtils.copyProperties(order, createOrderResultVO);
        // 订单关闭
        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                order.getId(),
                (msg -> {
                    msg.getMessageProperties().setHeader("x-delay", (3 * 60 * 60 * 1000) + "");
                    return msg;
                }));
        return createOrderResultVO;
    }

    @Override
    @Transactional
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#2#h",
            key = "T(String).valueOf(#createUserOrderDTO.userId)" +
                    ".concat('-').concat(#createUserOrderDTO.productId)" +
                    ".concat('-').concat(#createUserOrderDTO.paymentMethod)" +
                    ".concat('-').concat(#createUserOrderDTO.num)" +
                    ".concat('-').concat(#createUserOrderDTO.discountFlag)",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public Map<String, String> createUserOrder(CreateUserOrderDTO createUserOrderDTO) {
        UserVO userVO = userApi.getById(createUserOrderDTO.getUserId()).getResult();
        if (BeanUtil.isEmpty(userVO)) {
            log.error("创建订单，用户不存在,{}", createUserOrderDTO.getUserId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_NOT_EXIST);
        }
        ProductVO productVO = productApi.getProduct(createUserOrderDTO.getProductId()).getResult();
        if (BeanUtil.isEmpty(productVO)) {
            log.error("创建订单，产品不存在,{}", createUserOrderDTO.getProductId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        if (productVO.getPaymentModel() != 2) {
            log.error("创建订单，产品不是买断产品,{}", createUserOrderDTO.getProductId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_TYPE_ERROR);
        }
        // 创建order
        Order order = new Order();
        String readeNo = CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
        order.setTradeNo(readeNo);
        // 记录原价和实际付款价
        if (createUserOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createUserOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
            order.setPrice(productVO.getPrice());
            order.setCurrency("USD");
        } else {
            order.setPrice(productVO.getCnyPrice());
            order.setCurrency("CNY");
        }
        order.setReducedPrice(createUserOrderDTO.getPrice());
        order.setPayment(createUserOrderDTO.getPaymentMethod());
        order.setStatus(OrderConstant.CREATED);
        order.setType(1);
        order.setProductId(createUserOrderDTO.getProductId());
        order.setUserId(createUserOrderDTO.getUserId());
        order.setEmail(userVO.getEmail());
        order.setPayNumber(createUserOrderDTO.getNum());
        order.setDiscountType(createUserOrderDTO.getDiscountFlag());
        order.setSubscriptionType(0);
        order.setPaymentModel(2);
        order.setAppId(productVO.getAppId().toString());

        // 根据付款方式，发送http请求支付中台
        String configId;
        String currency;
        switch (createUserOrderDTO.getPaymentMethod()) {
            case 0:
                // paypal
                configId = payCenterConfigProperties.getPaypal();
                currency = "USD";
                break;
            case 1:
                // 支付宝
                configId = payCenterConfigProperties.getAlipay();
                currency = "CNY";
                break;
            case 2:
                // 微信
                configId = payCenterConfigProperties.getVx();
                currency = "CNY";
                break;
            case 3:
                // paddle
                configId = payCenterConfigProperties.getPaddle();
                currency = "USD";
                break;
            default:
                log.error("未知的支付方式，{}", createUserOrderDTO);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CREATE_ORDER_PARAMETER);
        }
        RSAUtils rsaUtils = new RSAUtils();
        String authorization = rsaUtils.encrypt(String.format("{\"configId\":\"%s\"}", configId));

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("orderNum", readeNo);
        jsonObject.put("currency", currency);
        jsonObject.put("amount", createUserOrderDTO.getPrice());
        jsonObject.put("goodName", productVO.getProductName());
        List<JSONObject> orderGoodsList = new ArrayList<>();
        JSONObject orderGood = new JSONObject();
        orderGood.put("unitPrice", createUserOrderDTO.getPrice());
        orderGood.put("num", 1);
        orderGoodsList.add(orderGood);
        jsonObject.put("orderGoodsList", orderGoodsList);

        // 请求支付中台
        log.info("创建订单，请求支付中台，{}", jsonObject);
        OkHttpClient client = new OkHttpClient().newBuilder()
                .connectTimeout(300, TimeUnit.SECONDS)
                .readTimeout(300, TimeUnit.SECONDS)
                .writeTimeout(300, TimeUnit.SECONDS)
                .build();
        okhttp3.MediaType mediaType = okhttp3.MediaType.parse("application/json");
        RequestBody body = RequestBody.create(mediaType, jsonObject.toString());
        Request request = new Request.Builder()
                .url(payCenterConfigProperties.getPayCenterUrl() + payCenterConfigProperties.getCreateOrderMethod())
                .method("POST", body)
                .addHeader("Authorization", authorization.replace("\r", "").replace("\n", ""))
                .addHeader("Content-Type", "application/json")
                .addHeader("Accept", "*/*")
                .addHeader("Connection", "keep-alive")
                .build();
        try {
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                // 解析响应
                String responseBody = response.body().string();
                JSONObject jsonResponse = JSON.parseObject(responseBody);
                String payHref = jsonResponse.getJSONObject("data").getString("payHref");
                order.setThirdOrderNo(jsonResponse.getJSONObject("data").getString("id"));
                order.setPagePayUrl(payHref);
                this.baseMapper.insert(order);
                // 订单关闭
                log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
                rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                        RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                        order.getId(),
                        (msg -> {
                            msg.getMessageProperties().setHeader("x-delay", (2 * 60 * 60 * 1000) + "");
                            return msg;
                        }));
                Map<String, String> result = new HashMap<>();
                result.put("payHref", payHref);
                result.put("orderId", order.getId());
                result.put("thirdOrderNo", order.getThirdOrderNo());
                return result;
            }
        } catch (IOException e) {
            log.error("请求支付中台创建订单失败", e);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        return null;
    }

    @Override
    @Transactional
    @Cacheable(value = RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER + "#2#h",
            key = "T(String).valueOf(#createUserOrderDTO.userId)" +
                    ".concat('-').concat(#createUserOrderDTO.productId)" +
                    ".concat('-').concat(#createUserOrderDTO.paymentMethod)" +
                    ".concat('-').concat(#createUserOrderDTO.num)" +
                    ".concat('-').concat(#createUserOrderDTO.discountFlag)",
            cacheManager = "myRedisCacheManager", unless = "#result == null")
    public Map<String, String> createUserSubscription(CreateUserOrderDTO createUserOrderDTO) {
        UserVO userVO = userApi.getById(createUserOrderDTO.getUserId()).getResult();
        if (BeanUtil.isEmpty(userVO)) {
            log.error("创建订阅，用户不存在,{}", createUserOrderDTO.getUserId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_USER_NOT_EXIST);
        }
        ProductVO productVO = productApi.getProduct(createUserOrderDTO.getProductId()).getResult();
        if (BeanUtil.isEmpty(productVO)) {
            log.error("创建订阅，产品不存在,{}", createUserOrderDTO.getProductId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_NOT_EXIST);
        }
        if (productVO.getPaymentModel() != 1) {
            log.error("创建订阅，产品不是订阅产品,{}", createUserOrderDTO.getProductId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRODUCT_TYPE_ERROR);
        }
        // 创建order
        Order order = new Order();
        String readeNo = CommonUtils.generateRightsId((long) ((Math.random() * 9999 + 1) * 10000));
        order.setTradeNo(readeNo);
        // 订阅目前只支持美元
        order.setPrice(productVO.getPrice());
        order.setCurrency("USD");
        if(ObjectUtils.isNotEmpty(createUserOrderDTO.getDiscountFlag()) && createUserOrderDTO.getDiscountFlag()!=0 && !createUserOrderDTO.getPrice().equals(productVO.getPrice())){
            order.setReducedPrice(createUserOrderDTO.getPrice());
        }
        order.setPayment(createUserOrderDTO.getPaymentMethod());
        order.setStatus(OrderConstant.CREATED);
        order.setType(1);
        order.setProductId(createUserOrderDTO.getProductId());
        order.setUserId(createUserOrderDTO.getUserId());
        order.setEmail(userVO.getEmail());
        order.setPayNumber(createUserOrderDTO.getNum());
        order.setDiscountType(createUserOrderDTO.getDiscountFlag());
        order.setSubscriptionType(1);
        order.setPaymentModel(1);
        order.setAppId(productVO.getAppId().toString());

        // 根据付款方式，发送http请求支付中台
        String configId;
        String currency = "USD";
        switch (createUserOrderDTO.getPaymentMethod()) {
            case 0:
                // paypal
                configId = payCenterConfigProperties.getPaypal();
                break;
/*            case 1:
                // 支付宝
                configId = payCenterConfigProperties.getAlipay();
                currency = "CNY";
                break;
            case 2:
                // 微信
                configId = payCenterConfigProperties.getVx();
                currency = "CNY";
                break;*/
            case 3:
                // paddle
                configId = payCenterConfigProperties.getPaddle();
                break;
            default:
                log.error("未知的支付方式，{}", createUserOrderDTO);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CREATE_ORDER_PARAMETER);
        }
        RSAUtils rsaUtils = new RSAUtils();
        String authorization = rsaUtils.encrypt(String.format("{\"configId\":\"%s\"}", configId));

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("orderNum", readeNo);
        jsonObject.put("currency", currency);
        // 订阅优惠类型 0 默认，1 首期优惠，2 试用（首期免费）
        int subDiscountFlag = 0;
        // 订阅周期付费，以产品价格为准
        if (createUserOrderDTO.getDiscountFlag() == 2) {
            // 需求更新，升级购买首期优惠改为每期都用优惠价
            jsonObject.put("price", order.getReducedPrice());
            subDiscountFlag = 3;
        } else {
            jsonObject.put("price", productVO.getPrice());
            // 设置订阅首期优惠
            if (order.getReducedPrice() != null) {
                subDiscountFlag = 1;
                jsonObject.put("discountPrice", order.getReducedPrice());
                if (order.getReducedPrice().compareTo(BigDecimal.ZERO) == 0) {
                    subDiscountFlag = 2;
                }
            }
        }
        jsonObject.put("discountFlag", subDiscountFlag);
        // 支付中台订阅周期:1 按月,2 按年；扣费周期根据产品 cycle 字段判断，支付中台目前没配置 按季度和半年
        if (productVO.getCycle() == 1) {
            jsonObject.put("cycle", 1);
        } else if (productVO.getCycle() == 4) {
            jsonObject.put("cycle", 2);
        }
        List<JSONObject> orderGoodsList = new ArrayList<>();
        JSONObject orderGood = new JSONObject();
        orderGood.put("unitPrice", createUserOrderDTO.getPrice());
        orderGood.put("num", 1);
        orderGoodsList.add(orderGood);
        jsonObject.put("orderGoodsList", orderGoodsList);

        // 请求支付中台
        log.info("创建订阅，请求支付中台，参数：{}", jsonObject);
        OkHttpClient client = new OkHttpClient().newBuilder()
                .connectTimeout(300, TimeUnit.SECONDS)
                .readTimeout(300, TimeUnit.SECONDS)
                .writeTimeout(300, TimeUnit.SECONDS)
                .build();
        okhttp3.MediaType mediaType = okhttp3.MediaType.parse("application/json");
        RequestBody body = RequestBody.create(mediaType, jsonObject.toString());
        Request request = new Request.Builder()
                .url(payCenterConfigProperties.getPayCenterUrl() + payCenterConfigProperties.getCreateSubscriptionMethod())
                .method("POST", body)
                .addHeader("Authorization", authorization.replace("\r", "").replace("\n", ""))
                .addHeader("Content-Type", "application/json")
                .addHeader("Accept", "*/*")
                .addHeader("Connection", "keep-alive")
                .build();
        try {
            Response response = client.newCall(request).execute();
            if (response.isSuccessful()) {
                // 解析响应
                String responseBody = response.body().string();
                JSONObject jsonResponse = JSON.parseObject(responseBody);
                String payHref = jsonResponse.getJSONObject("data").getString("payHref");
                order.setThirdOrderNo(jsonResponse.getJSONObject("data").getString("id"));
                order.setPagePayUrl(payHref);
                this.baseMapper.insert(order);
                // 订单关闭
                log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
                rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                        RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
                        order.getId(),
                        (msg -> {
                            msg.getMessageProperties().setHeader("x-delay", (2 * 60 * 60 * 1000) + "");
                            return msg;
                        }));
                Map<String, String> result = new HashMap<>();
                result.put("payHref", payHref);
                result.put("orderId", order.getId());
                result.put("thirdOrderNo", order.getThirdOrderNo());
                return result;
            }
        } catch (IOException e) {
            log.error("请求支付中台创建订阅失败", e);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        return null;
    }

    @Override
    @Transactional
    public CreateOrderResultVO appStoreCreateOrder(CreateOrderDTO createOrderDTO) {
        PaymentMethodEnum paymentMethodEnum = createOrderDTO.getPaymentMethod();
        log.info("createOrder:{}", createOrderDTO);
        Order order = new Order();
        order.setEmail(userApi.getById(createOrderDTO.getUserId()).getResult().getEmail());
//        order.setAppId(creatOrderDTO.getAppId());
//        order.setUserId(creatOrderDTO.getUserId());
//        order.setProductId(creatOrderDTO.getProductId());
        order.setAppId(createOrderDTO.getAppId());
        order.setUserId(createOrderDTO.getUserId());
        order.setProductId(createOrderDTO.getProductId());
        order.setPayment(paymentMethodEnum.getValue());
        order.setStatus(OrderConstant.CREATED);
        order.setInvoiceNo(MyDateUtils.getTimeStamp() + (int) ((Math.random() * 9 + 1) * 1000));
//        orders.setDiscount(createOrderDTO.getDiscount());
        String readeNo = MyDateUtils.getTimeStamp() + "-" + createOrderDTO.getProductId() + "-" + (int) ((Math.random() * 9 + 1) * 1000);
        order.setTradeNo(readeNo);
        //订单类型3appstore购买
        order.setType(3);
        this.baseMapper.insert(order);
        CreateOrderResultVO createOrderResultVO = new CreateOrderResultVO();
        createOrderResultVO.setUuid(UuidUtils.generateUuid());
        BeanUtils.copyProperties(order, createOrderResultVO);
//        // 订单关闭
//        log.info("rabbitMq 发送延时队列，关闭订单：{}", order.getId());
//        rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
//                RabbitMqConstant.PAYMENT_ORDER_ROUTING_KEY,
//                order.getId(),
//                (msg -> {
//                    msg.getMessageProperties().setHeader("x-delay", ( 3 * 60 * 60 * 1000) + "");
//                    return msg;
//                }));
        return createOrderResultVO;
    }

    @Override
    public void closeOrder(String orderId) {
        OrdersVO orders = getOrderById(orderId);
        if(ObjectUtils.isEmpty(orders)){
            return;
        }
        log.info("当前订单信息：{}", orders);
        if (ObjectUtils.isNotEmpty(orders.getStatus()) && orders.getStatus().equals(OrderConstant.CREATED)) {
            Order order = new Order();
            order.setStatus(OrderConstant.CLOSED);
            order.setId(orderId);
            this.baseMapper.updateById(order);
        }
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orders.getEmail() + "-" + orders.getProductId() + "-" + orders.getPayment());
        // 会员订单，通过userId
        deleteTheCacheInTheCacheManager(orders.getUserId() + "-" + orders.getProductId() + "-" + orders.getPayment() + "-" + orders.getPayNumber() + "-" + orders.getDiscountType());
        // 检查订单优惠券
        productApi.refundCouponByOrderId(orderId);
    }

    @Override
    public void closeOrderByUser(String userId, Integer platform) {
        List<OrdersVO> orderByUser = this.baseMapper.getOrderByUser(userId, platform);
        if(CollectionUtils.isEmpty(orderByUser)){
            return;
        }
        log.info("关闭会员用户相同订单：{}", orderByUser);
        for (OrdersVO orders : orderByUser) {
            log.info("当前订单信息：{}", orders);
            if (ObjectUtils.isNotEmpty(orders.getStatus()) && orders.getStatus().equals(OrderConstant.CREATED)) {
                Order order = new Order();
                order.setStatus(OrderConstant.CLOSED);
                order.setId(orders.getId());
                this.baseMapper.updateById(order);
            }
            //删除缓存中的订单
            deleteTheCacheInTheCacheManager(orders.getUserId() + "-" + orders.getProductId() + "-" + orders.getPayment() + "-" + orders.getPayNumber() + "-" + orders.getDiscountType());
            // 检查订单优惠券
            productApi.refundCouponByOrderId(orders.getId());
        }
    }

    @Override
    public OrderUpdateVO updateOrderSucceeded(OrderSucceededBO orderSucceeded) {
        Order orders = new Order();
        orders.setThirdOrderNo(orderSucceeded.getThirdOrderNo());
        orders.setThirdTradeNo(orderSucceeded.getThirdTradeNo());
        orders.setPrice(orderSucceeded.getPrice());
        orders.setUserId(orderSucceeded.getUserId());
        orders.setPayDate(new Date());
        orders.setSubscriptionId(orderSucceeded.getSubscriptionId());
        orders.setStatus(OrderConstant.COMPLETED);
        Order orderTradeNo;
        if (ObjectUtils.isNotEmpty(orderSucceeded.getOrderId())) {
            this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                    .eq(Order::getId, orderSucceeded.getOrderId()));
            orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                    .select(Order::getId, Order::getUserId, Order::getProductId, Order::getAppId, Order::getPayment, Order::getEmail)
                    .eq(Order::getId, orderSucceeded.getOrderId()));
        } else {
            this.baseMapper.update(orders, new LambdaQueryWrapper<Order>()
                    .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
            orderTradeNo = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>()
                    .select(Order::getId, Order::getUserId, Order::getProductId, Order::getAppId, Order::getPayment, Order::getEmail)
                    .eq(Order::getThirdTradeNo, orderSucceeded.getThirdTradeNo()));
        }
        //删除缓存中的订单
        deleteTheCacheInTheCacheManager(orderTradeNo.getEmail() + "-" + orderTradeNo.getProductId() + "-" + orderTradeNo.getPayment());
        return OrderUpdateVO.builder()
                .orderId(orderTradeNo.getId())
                .userId(orderTradeNo.getUserId())
                .productId(orderTradeNo.getProductId())
                .appId(orderTradeNo.getAppId())
                .build();
    }

    /**
     * 删除redis缓存管理器中的对应key的缓存
     *
     * @param key key
     */
    @Override
    public void deleteTheCacheInTheCacheManager(String key) {
        log.info("删除缓存中的订单 key" + key);
        RedisCache cache = (RedisCache) myRedisCacheManager.getCache(RedisKeyConstant.CACHE_PAYMENT_CREAT_ORDER);
        if (cache != null) {
            cache.evictIfPresent(key);
        }
    }

    @Override
    public OrdersVO getOrderById(String orderId) {
        OrdersVO ordersVO = new OrdersVO();
        Order orders = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId));
        if (ObjectUtils.isEmpty(orders)) {
            return null;
        }
        BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId)), ordersVO);
        return ordersVO;
    }

    @Override
    public Map<String, String> getStateByOrderId(String orderId) {
        Order order = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getId, orderId));
        if (ObjectUtils.isEmpty(order)) {
            return null;
        }
        log.info("获取订单状态，信息：{}", order);
        // 捕获PayPal买断订单，先不判断是否支付，兼容支付回调后修改了订单状态也需要捕获一次
        if (order.getPayment() == 0 && order.getPaymentModel() == 2) {
            log.info("paypal买断订单捕获");
            RSAUtils rsaUtils = new RSAUtils();
            String authorization = rsaUtils.encrypt(String.format("{\"configId\":\"%s\"}", payCenterConfigProperties.getPaypal()));

            OkHttpClient client = new OkHttpClient().newBuilder()
                    .connectTimeout(300, TimeUnit.SECONDS)
                    .readTimeout(300, TimeUnit.SECONDS)
                    .writeTimeout(300, TimeUnit.SECONDS)
                    .build();
            String url = payCenterConfigProperties.getPayCenterUrl() + payCenterConfigProperties.getCapturePaypalMethod() + "?tradeId=" + order.getThirdOrderNo();
            Request request = new Request.Builder()
                    .url(url)
                    .method("GET", null)
                    .addHeader("Authorization", authorization.replace("\r", "").replace("\n", ""))
                    .addHeader("Accept", "*/*")
                    .addHeader("Connection", "keep-alive")
                    .build();
            try {
                Response response = client.newCall(request).execute();
                if (response.isSuccessful()) {
                    // 解析响应
                    String responseBody = null;
                    if (response.body() != null) {
                        responseBody = response.body().string();
                    }
                    JSONObject data = JSON.parseObject(responseBody).getJSONObject("data");
                    if (ObjectUtils.isNotEmpty(data)) {
                        String status = data.getString("status");
                        if (StringUtils.isNotEmpty(status)) {
                            // 捕获订单
                            log.info("PayPal订单捕获：{}，状态：{}", order.getThirdOrderNo(), status);
                        }
                    }
                } else {
                    log.warn("捕获订单失败，{}", response.code());
                }
            } catch (IOException e) {
                log.warn("捕获订单异常，{}", e.getMessage());
            }
        }
        // 获取订单状态
        Map<String, String> map = new HashMap<>();
        if (!order.getStatus().equals(2)) {
            // 未支付订单，等待2s回调
            log.info("订单等待回调，{}", orderId);
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                log.error("线程中断", e);
            }
            map.put("status", "CREATED");
        } else {
            map.put("status", "COMPLETED");
        }
        map.put("payment", order.getPayment().toString());
        map.put("price", null != order.getReducedPrice() ? order.getReducedPrice().toEngineeringString() : order.getPrice().toEngineeringString());
        return map;
    }

    @Override
    public List<OrdersVO> getOrderListByStatus(Integer orderStatus, String userId) {
        // 获取订单
        LambdaQueryWrapper<Order> orderLambdaQueryWrapper = new LambdaQueryWrapper<>();
        orderLambdaQueryWrapper.eq(Order::getUserId, userId);
        if (orderStatus != null && orderStatus == 1) {
            orderLambdaQueryWrapper.eq(Order::getStatus, 0);
        }
        orderLambdaQueryWrapper.orderByDesc(Order::getCreatedAt);
        return baseMapper.selectList(orderLambdaQueryWrapper).stream().map(order -> {
            OrdersVO ordersVO = new OrdersVO();
            BeanUtils.copyProperties(order, ordersVO);
            return ordersVO;
        }).collect(Collectors.toList());
    }

    @Override
    public List<OrdersVO> getOrderByTradeNo(String tradeNo) {
        return this.baseMapper.selectOrdersByTradeNo(tradeNo);
    }
    @Override
    public ProductVO getProduct(String productId) {
        return this.baseMapper.getProduct(productId);
    }

    @Override
    public OrderUpdateVO createOrderManual(CreateOrderManualDTO createOrderManual) {
        Order orders = new Order();
        orders.setId(createOrderManual.getId());
        orders.setAppId(createOrderManual.getAppId());
        orders.setProductId(createOrderManual.getProductId());
        orders.setUserId(createOrderManual.getUserId());
        orders.setTradeNo(createOrderManual.getTradeNo());
        orders.setThirdTradeNo(createOrderManual.getThirdTradeNo());
        if (!StringUtils.isEmpty(createOrderManual.getThirdOrderNo())) {
            orders.setThirdOrderNo(createOrderManual.getThirdOrderNo());
        }
        orders.setEmail(createOrderManual.getEmail());
        orders.setSubscriptionId(createOrderManual.getSubscriptionId());
        orders.setPrice(createOrderManual.getPrice());
        orders.setPayment(createOrderManual.getPaymentMethod().getValue());
        orders.setStatus(OrderConstant.COMPLETED);
        orders.setPayDate(new Date());
        if(ObjectUtils.isNotEmpty(createOrderManual.getReducedPrice())){
            orders.setReducedPrice(createOrderManual.getReducedPrice());
        }
        //subscriptionType payNumber paymentModel 判断有没有，有的话才设置
        if (createOrderManual.getSubscriptionType() != null) {
            orders.setSubscriptionType(createOrderManual.getSubscriptionType());
        }
        if (createOrderManual.getPayNumber() != null) {
            orders.setPayNumber(createOrderManual.getPayNumber());
        }
        if (createOrderManual.getPaymentModel() != null) {
            orders.setPaymentModel(createOrderManual.getPaymentModel());
        }
        if (StringUtils.isNotEmpty(createOrderManual.getCurrency())) {
            orders.setCurrency(createOrderManual.getCurrency());
        }else {
            orders.setCurrency("USD");
        }
        if (createOrderManual.getResult() != null) {
            orders.setResult(createOrderManual.getResult());
        }
        this.baseMapper.insert(orders);
        return OrderUpdateVO.builder()
                .userId(createOrderManual.getUserId())
                .orderId(orders.getId())
                .build();
    }


    @Override
    public void chargebackOrder(@NotNull ChargebackOrderDTO chargebackOrderDTO) {
        log.info("手动退单修改状态：{}", chargebackOrderDTO);
        // 修改订单状态
        Order orders = new Order();
        orders.setId(chargebackOrderDTO.getOrderId());
        orders.setStatus(OrderConstant.REFUND);
        this.baseMapper.updateById(orders);
        log.info("手动退单修改状态成功：{}", chargebackOrderDTO);
    }

    @Override
    public void insertOrder(Order order) {
        this.baseMapper.insert(order);
    }

    @Override
    public List<OrdersVO> getOrderByUserId(String userId, String appId) {
        LambdaQueryWrapper<Order> orderLambdaQueryWrapper = new LambdaQueryWrapper<>();
        orderLambdaQueryWrapper.eq(Order::getUserId, userId).orderByDesc(Order::getCreatedAt);
        if (StringUtils.isNotEmpty(appId)) {
            orderLambdaQueryWrapper.eq(Order::getAppId, appId);
        }
        return this.baseMapper.selectList(orderLambdaQueryWrapper)
                .stream().map(order -> {
                    OrdersVO ordersVO = new OrdersVO();
                    BeanUtils.copyProperties(order, ordersVO);
                    return ordersVO;
                }).collect(Collectors.toList());
    }


    @Override
    public OrdersVO getOrderByThirdOrderNo(String thirdOrderNo) {
        OrdersVO ordersVO = new OrdersVO();
        Order order = this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getThirdOrderNo, thirdOrderNo));
        if (ObjectUtils.isNotEmpty(order)) {
            BeanUtils.copyProperties(this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getThirdOrderNo, thirdOrderNo)), ordersVO);
            return ordersVO;
        } else {
            return null;
        }
    }

    @Override
    public List<OrdersVO> getOrderByOrderNum(String orderNum) {
        return this.baseMapper.selectOrdersByOrderNum(orderNum);
    }

    @Override
    public List<OrdersVO> getOrderListByThirdOrderNo(String thirdOrderNo) {
        return this.baseMapper.selectOrdersByThirdOrderNo(thirdOrderNo);
    }

    @Override
    public Order getOrderByUserIdAndProductId(String userId, String productId) {
        return this.baseMapper.selectOne(new LambdaQueryWrapper<Order>().eq(Order::getUserId,userId)
                .eq(Order::getProductId,productId)
                .eq(Order::getStatus,2)
                .orderByDesc(Order::getPayDate).last("Limit 1"));
    }

}
