package cn.kdan.cloud.pdf.office.website.controller;

import cn.hutool.core.util.ObjectUtil;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.OrderService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/order")
@RequiredArgsConstructor
public class OrderController {

    private final OrderService orderService;

    /**
     * 创建订单，支持买断购买
     *
     * @param  createOrderDTO  createOrderDTO
     * @return String 支付链接
     */
    @PostMapping("/createOrder")
    public ResultMap<Map<String, String>> createOrder(@RequestBody CreateUserOrderDTO createOrderDTO) {
        return ResultMap.success(orderService.createOrder(createOrderDTO));
    }

    @PostMapping("/createSubscription")
    public ResultMap<Map<String, String>> createSubscription(@RequestBody CreateUserOrderDTO createOrderDTO) {
        return ResultMap.success(orderService.createSubscription(createOrderDTO));
    }

    /**
     * 根据订单id获取订单详情
     * @param orderId 订单id
     * @return OrdersVO
     */
    @GetMapping("/getStateByOrderId")
    public ResultMap<Map<String, String>> getStateByOrderId(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        Map<String, String> stateByOrderId = orderService.getStateByOrderId(orderId);
        if (ObjectUtil.isNotEmpty(stateByOrderId)) {
            return ResultMap.success(stateByOrderId);
        } else {
            return ResultMap.fail(null);
        }
    }

    @GetMapping("/getOrderListByStatus")
    public ResultMap<List<OrdersVO>> getOrderListByStatus(@Validated @NotNull(message = "订单状态不能为空") @RequestParam("orderStatus") Integer orderStatus) {
        return ResultMap.success(orderService.getOrderListByStatus(orderStatus));
    }

    @PostMapping("/closeOrder")
    public ResultMap<Boolean> closeOrder(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId) {
        return ResultMap.success(orderService.closeOrder(orderId));
    }

}
