package cn.kdan.cloud.pdf.office.website.controller;

import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderGiftLogVO;
import cn.kdan.cloud.pdf.office.api.product.feign.PrizeApi;
import cn.kdan.cloud.pdf.office.api.product.vo.PrizeVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.website.service.LotteryService;
import cn.kdan.cloud.pdf.office.website.service.OrderGiftService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.util.List;

/**
 * 会员赠送模块
 */
@RestController
@RequestMapping("/orderGift")
@RequiredArgsConstructor
public class OrderGiftController {
    private final OrderApi orderApi;
    private final OrderGiftService orderGiftService;

    /**
     * 判断用户有几个可赠送的会员
     */
    @GetMapping(value = "getGiftNum")
     public ResultMap<Integer> getGiftNum() {
         String userId = SecurityUtils.getUserId();
         return ResultMap.success(orderGiftService.getGiftNum(userId));
     }

    /**
     * 赠送
     * @param name 用户名
     * @param email 邮箱
     * @param isGift 是否强制赠送
     * @return
     */
    @PostMapping(value = "giveAway")
    public ResultMap<Boolean> giveAway(String name,String email,boolean isGift) {
        String userId = SecurityUtils.getUserId();
        orderGiftService.giveAway(userId,name,email,isGift);
        return ResultMap.success(Boolean.TRUE);
    }

    /**
     * 接收赠送
     * @param orderGiftLogId orderGiftLogId
     * @return
     */
    @PostMapping("/acceptUser")
    public ResultMap<Boolean> acceptUser(@RequestParam("orderGiftLogId") String orderGiftLogId) {
        orderGiftService.acceptUser(orderGiftLogId);
        return ResultMap.success();
    }

    /**
     * 赠送记录分页查询
     * @param email 邮箱
     * @param name 用户名
     * @param page 页码
     * @param pageSize 每页条数
     * @return
     */
    @GetMapping("/pageOrderGiftLog")
    public ResultMap<Page<OrderGiftLogVO>> pageOrderGiftLog(@RequestParam(name = "email",required = false)String email,
                                                            @RequestParam(name = "name",required = false)String name,
                                                            @RequestParam(name = "page",defaultValue = "1",required = false)Integer page,
                                                            @RequestParam(name ="pageSize",defaultValue = "10",required = false)Integer pageSize) {
        return orderApi.pageOrderGiftLog(SecurityUtils.getUserId(), email, name, page, pageSize);
    }

}
