package cn.kdan.cloud.pdf.office.website.service;

import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.website.dto.IsSubscriptionDTO;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/2/9
 * @eo.api-type http
 * @eo.groupName 默认分组
 * @eo.path
 */
public interface PaymentService {

    /**
     * 获取订单记录
     *
     * @return SubscriptionSchemeVO
     */
    ResultMap<Map<String,SubscriptionSchemeVO>> getSubscriptionSchemeInfo(String appId);

    /**
     * 获取订阅结束时间
     *
     * @param platform 平台
     * @return LocalDateTime
     */
    LocalDateTime getSubscriptionEndDate(Integer platform);

    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    UserInfoVO appStoreEquityVerification(EquityVerificationDTO equityVerificationDTO);

    /**
     * 谷歌权益校验
     * @param googlePayDTO
     * @return
     */
    UserInfoVO googleEquityVerification(GooglePayDTO googlePayDTO);

    /**
     * 判断是否有订阅
     *
     * @param isSubscriptionDTO isSubscriptionDTO
     * @return Boolean true: 折扣，false：原价
     */
    boolean isSubscription(IsSubscriptionDTO isSubscriptionDTO);

    CreateOrderResultVO createOrder(CreateOrderDTO createOrderDTO);

    /**
     * 取消订阅
     * @param productId 产品id
     * @return
     */
    void cancelSubscription(String productId);

    /**
     * 创建订单（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    CreateOrderResultVO appStoreCreateOrder(CreateOrderDTO createOrderDTO);
}
