package cn.kdan.cloud.pdf.office.system.controller;

import cn.kdan.cloud.pdf.office.api.account.dto.LogOffUserDTO;
import cn.kdan.cloud.pdf.office.api.account.dto.PermissionGiftDTO;
import cn.kdan.cloud.pdf.office.api.account.dto.PermissionTransferDTO;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.account.vo.UserDetailVO;
import cn.kdan.cloud.pdf.office.api.account.vo.UserPageVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import java.security.Principal;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private UserApi userApi;

    /**
     * 分页查询
     *
     * @param selectType 筛选类型 正选1 反选2
     * @param appIds 产品id
     * @param roles 账号身份
     * @param accountTypes 账号类型
     * @param accountSources 账号来源
     * @param payTypes 付费类型
     * @param queryTimeType 查询时间类型 1 创建时间 2更新时间
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryString 查询string id或者邮箱
     * @return PageInfo
     */
    @GetMapping("/page")
    public ResultMap<PageInfo<UserPageVO>> page(@RequestParam(required = false, defaultValue = "1") Integer page,
                                                @RequestParam(required = false, defaultValue = "10") Integer pageSize,
                                                @RequestParam(required = false) String selectType,
                                                @RequestParam(required = false)  List<String> appIds,
                                                @RequestParam(required = false)  List<String> roles,
                                                @RequestParam(required = false)  List<String> accountTypes,
                                                @RequestParam(required = false)  List<String> accountSources,
                                                @RequestParam(required = false)  List<String> payTypes,
                                                @RequestParam(required = false)  String queryTimeType,
                                                @RequestParam(required = false)  Date startTime,
                                                @RequestParam(required = false)  Date endTime,
                                                @RequestParam(required = false)  List<String> queryString) {
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,userApi.page(page,pageSize,selectType,appIds,roles,accountTypes,accountSources,payTypes,queryTimeType,startTime,endTime,queryString).getResult());
    }

    /**
     * 运营后台查看账号详情
     *
     * @return ResultMap
     */
    @GetMapping("/detail")
    public ResultMap<UserDetailVO> detail(Principal principal) {
        String id = me(principal);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,userApi.detail(id).getResult());
    }

    /**
     * 后台强制注销用户
     *
     * @param logOffUserDTO
     */
    @PostMapping("/logOff")
    public ResultMap<Boolean> logOff(@RequestBody LogOffUserDTO logOffUserDTO) {
        userApi.logOff(logOffUserDTO);
        return new ResultMap<>(CommonConstant.SUCCESS, CommonConstant.RESULT_SUCCESS,Boolean.TRUE);
    }


    /**
     * 返回用户id
     *
     * @param principal 凭证
     * @return
     */
    public String me(Principal principal) {
        if (principal instanceof OAuth2Authentication) {
            {
                return ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal().toString();
            }
        }
        return null;
    }

}
